@@ -2,19 +2,17 @@
 #  Copyright (c) 1995-2001, Raphael Manfredi
 #  Copyright (c) 2002-2014 by the Perl 5 Porters
 #  Copyright (c) 2015-2016 cPanel Inc
-#  Copyright (c) 2017 Reini Urban
 #
 #  You may redistribute only under the same terms as Perl 5, as specified
 #  in the README file that comes with the distribution.
 #
 
-BEGIN { require XSLoader }
+require XSLoader;
 require Exporter;
-package Storable;
+package Storable; @ISA = qw(Exporter);
 
-our @ISA = qw(Exporter);
-our @EXPORT = qw(store retrieve);
-our @EXPORT_OK = qw(
+@EXPORT = qw(store retrieve);
+@EXPORT_OK = qw(
 	nstore store_fd nstore_fd fd_retrieve
 	freeze nfreeze thaw
 	dclone
@@ -22,24 +20,12 @@ our @EXPORT_OK = qw(
 	lock_store lock_nstore lock_retrieve
         file_magic read_magic
 	BLESS_OK TIE_OK FLAGS_COMPAT
-        stack_depth stack_depth_hash
 );
 
-our ($canonical, $forgive_me);
+use vars qw($canonical $forgive_me $VERSION);
 
-BEGIN {
-  our $VERSION = '3.24_50';
-}
-
-our $recursion_limit;
-our $recursion_limit_hash;
-
-$recursion_limit = 512
-  unless defined $recursion_limit;
-$recursion_limit_hash = 256
-  unless defined $recursion_limit_hash;
-
-use Carp;
+$VERSION = '3.05';
+#$VERSION = eval $VERSION;
 
 BEGIN {
     if (eval {
@@ -56,23 +42,16 @@ BEGIN {
     # provide a fallback implementation.
     #
     unless ($Storable::{logcroak} && *{$Storable::{logcroak}}{CODE}) {
-        *logcroak = \&Carp::croak;
-    }
-    else {
-        # Log::Agent's logcroak always adds a newline to the error it is
-        # given.  This breaks refs getting thrown.  We can just discard what
-        # it throws (but keep whatever logging it does) and throw the original
-        # args.
-        no warnings 'redefine';
-        my $logcroak = \&logcroak;
+        require Carp;
         *logcroak = sub {
-            my @args = @_;
-            eval { &$logcroak };
-            Carp::croak(@args);
+            Carp::croak(@_);
         };
     }
     unless ($Storable::{logcarp} && *{$Storable::{logcarp}}{CODE}) {
-        *logcarp = \&Carp::carp;
+	require Carp;
+        *logcarp = sub {
+          Carp::carp(@_);
+        };
     }
 }
 
@@ -106,12 +85,22 @@ $Storable::flags = FLAGS_COMPAT;
 $Storable::downgrade_restricted = 1;
 $Storable::accept_future_minor = 1;
 
-BEGIN { XSLoader::load('Storable') };
+XSLoader::load('Storable', $Storable::VERSION);
 
 #
 # Determine whether locking is possible, but only when needed.
 #
 
+my $CAN_FLOCK;
+BEGIN {
+    require Config;
+    $CAN_FLOCK =
+      $Config::Config{'d_flock'} ||
+      $Config::Config{'d_fcntl_can_lock'} ||
+      $Config::Config{'d_lockf'};
+}
+sub CAN_FLOCK () { $CAN_FLOCK } 
+
 sub show_file_magic {
     print <<EOM;
 #
@@ -266,7 +255,7 @@ sub _store {
     local *FILE;
     if ($use_locking) {
         open(FILE, ">>", $file) || logcroak "can't write into $file: $!";
-        unless (CAN_FLOCK) {
+        unless (&CAN_FLOCK) {
             logcarp
               "Storable::lock_store: fcntl/flock emulation broken on $^O";
             return undef;
@@ -292,10 +281,7 @@ sub _store {
     if (!(close(FILE) or undef $ret) || $@) {
         unlink($file) or warn "Can't unlink $file: $!\n";
     }
-    if ($@) {
-        $@ =~ s/\.?\n$/,/ unless ref $@;
-        logcroak $@;
-    }
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
     $@ = $da;
     return $ret;
 }
@@ -368,10 +354,7 @@ sub _freeze {
     my $ret;
     # Call C routine mstore or net_mstore, depending on network order
     eval { $ret = &$xsptr($self) };
-    if ($@) {
-        $@ =~ s/\.?\n$/,/ unless ref $@;
-        logcroak $@;
-    }
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
     $@ = $da;
     return $ret ? $ret : undef;
 }
@@ -410,7 +393,7 @@ sub _retrieve {
     my $self;
     my $da = $@;			# Could be from exception handler
     if ($use_locking) {
-        unless (CAN_FLOCK) {
+        unless (&CAN_FLOCK) {
             logcarp
               "Storable::lock_store: fcntl/flock emulation broken on $^O";
             return undef;
@@ -420,10 +403,7 @@ sub _retrieve {
     }
     eval { $self = pretrieve($FILE, $flags) };		# Call C routine
     close($FILE);
-    if ($@) {
-        $@ =~ s/\.?\n$/,/ unless ref $@;
-        logcroak $@;
-    }
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
     $@ = $da;
     return $self;
 }
@@ -441,10 +421,7 @@ sub fd_retrieve {
     my $self;
     my $da = $@;				# Could be from exception handler
     eval { $self = pretrieve($file, $flags) };	# Call C routine
-    if ($@) {
-        $@ =~ s/\.?\n$/,/ unless ref $@;
-        logcroak $@;
-    }
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
     $@ = $da;
     return $self;
 }
@@ -469,61 +446,11 @@ sub thaw {
     my $self;
     my $da = $@;			        # Could be from exception handler
     eval { $self = mretrieve($frozen, $flags) };# Call C routine
-    if ($@) {
-        $@ =~ s/\.?\n$/,/ unless ref $@;
-        logcroak $@;
-    }
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
     $@ = $da;
     return $self;
 }
 
-#
-# _make_re($re, $flags)
-#
-# Internal function used to thaw a regular expression.
-#
-
-my $re_flags;
-BEGIN {
-    if ($] < 5.010) {
-        $re_flags = qr/\A[imsx]*\z/;
-    }
-    elsif ($] < 5.014) {
-        $re_flags = qr/\A[msixp]*\z/;
-    }
-    elsif ($] < 5.022) {
-        $re_flags = qr/\A[msixpdual]*\z/;
-    }
-    else {
-        $re_flags = qr/\A[msixpdualn]*\z/;
-    }
-}
-
-sub _make_re {
-    my ($re, $flags) = @_;
-
-    $flags =~ $re_flags
-        or die "regexp flags invalid";
-
-    my $qr = eval "qr/\$re/$flags";
-    die $@ if $@;
-
-    $qr;
-}
-
-if ($] < 5.012) {
-    eval <<'EOS'
-sub _regexp_pattern {
-    my $re = "" . shift;
-    $re =~ /\A\(\?([xism]*)(?:-[xism]*)?:(.*)\)\z/s
-        or die "Cannot parse regexp /$re/";
-    return ($2, $1);
-}
-1
-EOS
-      or die "Cannot define _regexp_pattern: $@";
-}
-
 1;
 __END__
 
@@ -726,22 +653,6 @@ all values unlocked.  To make Storable C<croak()> instead, set
 C<$Storable::downgrade_restricted> to a C<FALSE> value.  To restore
 the default set it back to some C<TRUE> value.
 
-The cperl PERL_PERTURB_KEYS_TOP hash strategy has a known problem with
-restricted hashes.
-
-=item huge objects
-
-On 64bit systems some data structures may exceed the 2G (i.e. I32_MAX)
-limit. On 32bit systems also strings between I32 and U32 (2G-4G).
-Since Storable 3.00 (not in perl5 core) we are able to store and
-retrieve these objects, even if perl5 itself is not able to handle
-them.  These are strings longer then 4G, arrays with more then 2G
-elements and hashes with more then 2G elements. cperl forbids hashes
-with more than 2G elements, but this fail in cperl then. perl5 itself
-at least until 5.26 allows it, but cannot iterate over them.
-Note that creating those objects might cause out of memory
-exceptions by the operating system before perl has a chance to abort.
-
 =item files from future versions of Storable
 
 Earlier versions of Storable would immediately croak if they encountered
@@ -768,10 +679,10 @@ relevant feature.
 
 =head1 ERROR REPORTING
 
-Storable uses the "exception" paradigm, in that it does not try to
-workaround failures: if something bad happens, an exception is
-generated from the caller's perspective (see L<Carp> and C<croak()>).
-Use eval {} to trap those exceptions.
+Storable uses the "exception" paradigm, in that it does not try to workaround
+failures: if something bad happens, an exception is generated from the
+caller's perspective (see L<Carp> and C<croak()>).  Use eval {} to trap
+those exceptions.
 
 When Storable croaks, it tries to report the error via the C<logcroak()>
 routine from the C<Log::Agent> package, if it is available.
@@ -779,13 +690,6 @@ routine from the C<Log::Agent> package, if it is available.
 Normal errors are reported by having store() or retrieve() return C<undef>.
 Such errors are usually I/O errors (or truncated stream errors at retrieval).
 
-When Storable throws the "Max. recursion depth with nested structures
-exceeded" error we are already out of stack space. Unfortunately on
-some earlier perl versions cleaning up a recursive data structure
-recurses into the free calls, which will lead to stack overflows in
-the cleanup. This data structure is not properly cleaned up then, it
-will only be destroyed during global destruction.
-
 =head1 WIZARDS ONLY
 
 =head2 Hooks
@@ -948,49 +852,6 @@ There are a few things you need to know, however:
 
 =item *
 
-From Storable 3.05 to 3.13 we probed for the stack recursion limit for references,
-arrays and hashes to a maximal depth of ~1200-35000, otherwise we might
-fall into a stack-overflow.  On JSON::XS this limit is 512 btw.  With
-references not immediately referencing each other there's no such
-limit yet, so you might fall into such a stack-overflow segfault.
-
-This probing and the checks we performed have some limitations:
-
-=over
-
-=item *
-
-the stack size at build time might be different at run time, eg. the
-stack size may have been modified with ulimit(1).  If it's larger at
-run time Storable may fail the freeze() or thaw() unnecessarily.  If
-it's larger at build time Storable may segmentation fault when
-processing a deep structure at run time.
-
-=item *
-
-the stack size might be different in a thread.
-
-=item *
-
-array and hash recursion limits are checked separately against the
-same recursion depth, a frozen structure with a large sequence of
-nested arrays within many nested hashes may exhaust the processor
-stack without triggering Storable's recursion protection.
-
-=back
-
-So these now have simple defaults rather than probing at build-time.
-
-You can control the maximum array and hash recursion depths by
-modifying C<$Storable::recursion_limit> and
-C<$Storable::recursion_limit_hash> respectively.  Either can be set to
-C<-1> to prevent any depth checks, though this isn't recommended.
-
-If you want to test what the limits are, the F<stacksize> tool is
-included in the C<Storable> distribution.
-
-=item *
-
 You can create endless loops if the things you serialize via freeze()
 (for instance) point back to the object we're trying to serialize in
 the hook.
@@ -1003,12 +864,6 @@ B, and if there is a serializing hook in A that says freeze(B), then when
 deserializing, we'll get [A', C'] where A' refers to B', but C' refers to D,
 a deep clone of B'.  The topology was not preserved.
 
-=item *
-
-The maximal stack recursion limit for your system is returned by
-C<stack_depth()> and C<stack_depth_hash()>. The hash limit is usually
-half the size of the array and ref limit, as the Perl hash API is not optimal.
-
 =back
 
 That's why C<STORABLE_freeze> lets you provide a list of references
@@ -1152,43 +1007,43 @@ such.
 
 Here are some code samples showing a possible usage of Storable:
 
- use Storable qw(store retrieve freeze thaw dclone);
+	use Storable qw(store retrieve freeze thaw dclone);
 
- %color = ('Blue' => 0.1, 'Red' => 0.8, 'Black' => 0, 'White' => 1);
+	%color = ('Blue' => 0.1, 'Red' => 0.8, 'Black' => 0, 'White' => 1);
 
- store(\%color, 'mycolors') or die "Can't store %a in mycolors!\n";
+	store(\%color, 'mycolors') or die "Can't store %a in mycolors!\n";
 
- $colref = retrieve('mycolors');
- die "Unable to retrieve from mycolors!\n" unless defined $colref;
- printf "Blue is still %lf\n", $colref->{'Blue'};
+	$colref = retrieve('mycolors');
+	die "Unable to retrieve from mycolors!\n" unless defined $colref;
+	printf "Blue is still %lf\n", $colref->{'Blue'};
 
- $colref2 = dclone(\%color);
+	$colref2 = dclone(\%color);
 
- $str = freeze(\%color);
- printf "Serialization of %%color is %d bytes long.\n", length($str);
- $colref3 = thaw($str);
+	$str = freeze(\%color);
+	printf "Serialization of %%color is %d bytes long.\n", length($str);
+	$colref3 = thaw($str);
 
 which prints (on my machine):
 
- Blue is still 0.100000
- Serialization of %color is 102 bytes long.
+	Blue is still 0.100000
+	Serialization of %color is 102 bytes long.
 
 Serialization of CODE references and deserialization in a safe
 compartment:
 
 =for example begin
 
- use Storable qw(freeze thaw);
- use Safe;
- use strict;
- my $safe = new Safe;
+	use Storable qw(freeze thaw);
+	use Safe;
+	use strict;
+	my $safe = new Safe;
         # because of opcodes used in "use strict":
- $safe->permit(qw(:default require));
- local $Storable::Deparse = 1;
- local $Storable::Eval = sub { $safe->reval($_[0]) };
- my $serialized = freeze(sub { 42 });
- my $code = thaw($serialized);
- $code->() == 42;
+	$safe->permit(qw(:default require));
+	local $Storable::Deparse = 1;
+	local $Storable::Eval = sub { $safe->reval($_[0]) };
+	my $serialized = freeze(sub { 42 });
+	my $code = thaw($serialized);
+	$code->() == 42;
 
 =for example end
 
@@ -1213,23 +1068,21 @@ in the value of a hash key that is overridden by another key/value
 pair in the same hash, thus causing immediate destructor execution.
 
 To disable blessing objects while thawing/retrieving remove the flag
-C<BLESS_OK> = 2 from C<$Storable::flags> or set the 2nd argument for
+BLESS_OK = 2 from C<$Storable::flags> or set the 2nd argument for
 thaw/retrieve to 0.
 
-To disable tieing data while thawing/retrieving remove the flag C<TIE_OK>
+To disable tieing data while thawing/retrieving remove the flag TIE_OK
 = 4 from C<$Storable::flags> or set the 2nd argument for thaw/retrieve
 to 0.
 
-With the default setting of C<$Storable::flags> = 6, creating or destroying
+With the default setting of $Storable::flags = 6, creating or destroying
 random objects, even renamed objects can be controlled by an attacker.
 See CVE-2015-1592 and its metasploit module.
 
-If your application requires accepting data from untrusted sources,
-you are best off with a less powerful and more-likely safe
-serialization format and implementation. If your data is sufficiently
-simple, L<Cpanel::JSON::XS>, L<Data::MessagePack> or L<Sereal> are the best
-choices and offer maximum interoperability, but note that Sereal is
-L<unsafe by default|Sereal::Decoder/ROBUSTNESS>.
+If your application requires accepting data from untrusted sources, you
+are best off with a less powerful and more-likely safe serialization format
+and implementation. If your data is sufficiently simple, JSON is the best
+choice and offers maximum interoperability.
 
 =head1 WARNING
 
@@ -1257,42 +1110,9 @@ populated, sorted and freed.  Some tests have shown a halving of the
 speed of storing -- the exact penalty will depend on the complexity of
 your data.  There is no slowdown on retrieval.
 
-=head1 REGULAR EXPRESSIONS
-
-Storable now has experimental support for storing regular expressions,
-but there are significant limitations:
-
-=over
-
-=item *
-
-perl 5.8 or later is required.
-
-=item *
-
-regular expressions with code blocks, ie C</(?{ ... })/> or C</(??{
-... })/> will throw an exception when thawed.
-
-=item *
-
-regular expression syntax and flags have changed over the history of
-perl, so a regular expression that you freeze in one version of perl
-may fail to thaw or behave differently in another version of perl.
-
-=item *
-
-depending on the version of perl, regular expressions can change in
-behaviour depending on the context, but later perls will bake that
-behaviour into the regexp.
-
-=back
-
-Storable will throw an exception if a frozen regular expression cannot
-be thawed.
-
 =head1 BUGS
 
-You can't store GLOB, FORMLINE, etc.... If you can define semantics
+You can't store GLOB, FORMLINE, REGEXP, etc.... If you can define semantics
 for those operations, feel free to enhance Storable so that it can
 deal with them.
 
@@ -1333,6 +1153,34 @@ to string or floating point.  In other words values that had been generated
 by integer operations such as logic ops and then not used in any string or
 arithmetic context before storing.
 
+=head2 Large data on 64-bit platforms
+
+Storable's current data format is incapable of representing lengths greater
+than fit into a signed 32-bit integer, or about 2 GB. In practice, this
+means that, even with the latest Perl and a 64-bit machine with plenty of
+memory, you cannot store, retrieve, or clone any of the following:
+
+=over 4
+
+=item *
+
+A string containing 2**31 or more bytes (including as an element of an array, or
+a key or value in a hash)
+
+=item *
+
+An array with 2**31 or more elements
+
+=item *
+
+A hash with 2**31 or more keys
+
+=back
+
+Attempting to do so will yield an exception.
+
+This may be fixed in the future.
+
 =head2 64 bit data in perl 5.6.0 and 5.6.1
 
 This section only applies to you if you have existing data written out
@@ -1406,7 +1254,7 @@ Thank you to (in chronological order):
 	Dominic Dunlop <domo@computer.org>
 	Erik Haugan <erik@solbors.no>
 	Benjamin A. Holzman <ben.holzman@grantstreet.com>
-	Reini Urban <rurban@cpan.org>
+	Reini Urban <rurban@cpanel.net>
 	Todd Rinaldo <toddr@cpanel.net>
 	Aaron Crane <arc@cpan.org>
 
@@ -1423,14 +1271,13 @@ Murray Nesbitt made Storable thread-safe.  Marc Lehmann added overloading
 and references to tied items support.  Benjamin Holzman added a performance
 improvement for overloaded classes; thanks to Grant Street Group for footing
 the bill.
-Reini Urban took over maintenance from p5p, and added security fixes
-and huge object support.
 
 =head1 AUTHOR
 
 Storable was written by Raphael Manfredi
 F<E<lt>Raphael_Manfredi@pobox.comE<gt>>
-Maintenance is now done by cperl L<http://perl11.org/cperl>
+Maintenance is now done by the perl5-porters
+F<E<lt>perl5-porters@perl.orgE<gt>>
 
 Please e-mail us with problems, bug fixes, comments and complaints,
 although if you have compliments you should send them to Raphael.
